--- Guilder: Raid Manager Module
-- This module allows the user to manage the members' Raid
-- @class file
-- @name Modules/Raid.lua
-- @release 2.0.x

--[[
   Copyright 2009 Christo Kotze (onaforeignshore)

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
]]

if select(6, GetAddOnInfo("Guilder_" .. (debugstack():match("[o%.][d%.][u%.]les\\(.-)\\") or ""))) ~= "MISSING" then return end

local Guilder = _G.Guilder
if not Guilder then
  error("Guilder_Raid requires Guilder")
end
local L = Guilder.L

local DEBUG = Guilder.DEBUG
local expect = Guilder.expect

-- default settings for the addon -------------------------------------------

local DATABASE_DEFAULTS = {
	announceRoll = false,
	includeOffline = false,
	allowAssist = false,
}

-- declare the module -------------------------------------------------------

local Guilder_Raid = Guilder:NewModule("Raid", "AceEvent-3.0", "AceHook-3.0")
Guilder_Raid:SetName(L["Raid Wizard"])
Guilder_Raid:SetDescription(L["Various Raid related utilities."])
Guilder_Raid:SetDefaults(DATABASE_DEFAULTS)
if DEBUG then
	Guilder_Raid:SetIcon([[Interface\AddOns\Guilder\Modules\Raid\Raid]])
else
	Guilder_Raid:SetIcon([[Interface\AddOns\Guilder_Raid\Raid]])
end

-- variables ----------------------------------------------------------------

-- Standard Module Methods --------------------------------------------------

-- Is called when the module is loaded
-- @usage internal use only
function Guilder_Raid:OnModuleLoaded(module)
	if not self.db.profile.enabled then return end
	local id = module.id
end

-- Is called when the module is enabled
-- @usage internal use only
function Guilder_Raid:OnEnable()
	LibGroup.RegisterCallback(self, "AssistAdded", "RefreshConfig")
	LibGroup.RegisterCallback(self, "AssistRemoved", "RefreshConfig")
	LibGroup.RegisterCallback(self, "LeaderChanged", "RefreshConfig")
	LibGroup.RegisterCallback(self, "MasterLooterChanged", "RefreshConfig")
end

-- Is called when the module is disabled
-- @usage internal use only
function Guilder_Raid:OnDisable()
end

-- Is called when the user opens the configuration screen
-- @usage internal use only
function Guilder_Raid:UpdateConfigOptions()
	if Guilder.configLoaded then
		self.options.div.args.raidRoll.disabled = not (Guilder:IsInRaid() and (LibGroup:GetRank() > 0 or LibGroup:IsMasterLooter()))
		if Guilder:IsInRaid() then
			if LibGroup:GetRank() == 2 then
				self.options.div.args.h2 = {
					type = "header",
					name = L["Ready Check Request"],
					order = 5,
				}
				self.options.div.args.readyCheck = {
					type = "execute",
					name = L["Ready Check"],
					desc = L["Do a Ready Check"],
					order = 6,
					func = function() self:ReadyCheck() end
				}
				if Guilder:IsInRaid() then
					self.options.div.args.allowAssist = {
						type = "toggle",
						name = L["Allow Assist Requests"],
						desc = L["Automatically do a Ready Check when Raid Assistant requests one?"],
						order = 7,
						get = function() return self.db.profile.allowAssist end,
						set = function(info, v) self.db.profile.allowAssist = v end
					}
				else
					self.options.div.args.allowAssist = nil
				end
			elseif LibGroup:GetRank() == 1 then
				self.options.div.args.h2 = {
					type = "header",
					name = L["Ready Check Request"],
					order = 5,
				}
				self.options.div.args.readyCheck = {
					type = "execute",
					name = L["Request Ready Check"],
					desc = L["Request a Ready Check from the group leader"],
					order = 6,
					func = function() self:ReadyCheck() end
				}
				self.options.div.args.allowAssist = nil
			else
				self.options.div.args.h2 = nil
				self.options.div.args.readyCheck = nil
				self.options.div.args.allowAssist = nil
			end
		end
	end
end

-- Is called when the module receives a message
-- @usage internal use only
function Guilder_Raid:CommHandler(distribution, sender, command, ...)
	-- if module is disabled, do nothing
	if not self.db.profile.enabled then
		if command == "URRC" then
			Guilder:ShowMessage(L["%s tried to request a ready check, but Raid Wizard was disabled."]:format(sender))
			Guilder:SendComm(self.id, "WHISPER", sender, "RRRC", false)
		end
		return
	end
	-- only handle comms if they are not from yourself
	if sender == myName then return end
	local t = {}
	local n = select('#', ...)
	-- Update: Request Ready Check
	if command == "URRC" then
		Guilder:ShowMessage(L["%s requested a Ready Check"]:format(sender))
		Guilder:SendComm(self.id, "WHISPER", sender, "RRRC", true)
		-- do a ready check if allowed
		if self.db.profile.allowAssist() then
			DoReadyCheck()
		end
	end
	-- Response: Request Ready Check
	if command == "RRRC" then
		if n == 1 then
			if select(1, ...) then
				Guilder:ShowMessage(L["%s has received your Ready Check request"]:format(sender))
			else
				Guilder:ShowMessage(L["%s's Raid Wizard is disabled! Ready Check request has been declined."]:format(sender))
			end
		end
	end
end

function Guilder_Raid:RefreshConfig()
	LibStub("AceConfigRegistry-3.0"):NotifyChange("Guilder")
end

-- General functions --------------------------------------------------------

function Guilder_Raid:RaidRoll()
	if not self.db.profile.enabled then
		Guilder:ShowMessage(L["%s is disabled"]:format("|cff00c0ff"..L["Raid Wizard"].."|r"))
		return
	end
	if self.db.profile.announceRoll then
		Guilder:SendMessage(L["%s is Raid Rolling! Check the raid channel for the result."]:format(myName), "RAID_WARNING")
	end
	local t = {}
	for k, _ in LibGroup:Iterate(self.db.profile.includeOffline) do
		t[#t + 1] = k
	end
	local r = random(#t)
	local m
	for i = 1, #t, 5 do
		m = i..". "..t[i]
		if t[i + 1] then m = m.."   "..(i + 1)..". "..t[i + 1] end
		if t[i + 2] then m = m.."   "..(i + 2)..". "..t[i + 2] end
		if t[i + 3] then m = m.."   "..(i + 3)..". "..t[i + 3] end
		if t[i + 4] then m = m.."   "..(i + 4)..". "..t[i + 4] end
		Guilder:SendMessage(m, "RAID")
	end
	Guilder:SendMessage(L["Rolled %s. %s wins!"]:format(r, t[r]), "RAID")
end

function Guilder_Raid:ReadyCheck()
	if LibGroup:GetRank() == 2 then
		DoReadyCheck()
	else
		Guilder:SendComm(self.id, "WHISPER", LibGroup:GetGroupLeader(), "URRC")
	end
end
